<?php
/**
 * @package CSS_Mega_Menu
 * @version 1.0.0
 */
/*
Plugin Name: CSS Mega Menu
Plugin URI: http://codecanyon.net/user/Keliah/portfolio
Description: CSS Mega Menu is implementing a highly configurable, menu containing fully customizable drop-down areas. The content of these drop-down areas can be edited just like any other page or post. The plugin also packages some cool shortcodes that will make your live easier while editing the menu content. <strong>License: This plugin comes with two types of commercial licenses. Read more about the available options here: <a href="http://codecanyon.net/wiki/support/legal-terms/licensing-terms/">http://codecanyon.net/wiki/support/legal-terms/licensing-terms/</a></strong>
Author: Cristian
Version: 1.0
Author URI: http://codecanyon.net/user/o_creative/profile
License: This plugin comes with two types of commercial license. Read more about the available options here: http://codecanyon.net/wiki/support/legal-terms/licensing-terms/
*/

/*************************************************************************************/
/*                                                                                   */
/*                       CREATE ADMIN INTERFACE AND OPTIONS                          */
/*                                                                                   */
/*************************************************************************************/


//Build the admin interface by adding a new custom post type for the menu
add_action('init', 'cmm_init');
function cmm_init() 
{
  $labels = array(
    'name' => _x('Mega Menu', 'post type general name'),
    'singular_name' => _x('Mega Menu', 'post type singular name'),
    'add_new' => _x('Add New Menu Item', 'cmm'),
    'add_new_item' => __('Add New Menu Item'),
    'edit_item' => __('Edit Menu Item'),
    'new_item' => __('New Menu Item'),
    'view_item' => __('View Menu Item'),
    'search_items' => __('Search Menu Items'),
    'not_found' =>  __('No menu item found'),
    'not_found_in_trash' => __('No menu items found in Trash'), 
    'parent_item_colon' => '',
    'menu_name' => 'Mega Menu'

  );
  $args = array(
    'label' => __('Menus'),
    'labels' => $labels,
	'singular_label' => __('Menu'),
    'public' => true,
    'publicly_queryable' => true,
    'show_ui' => true, 
    'show_in_menu' => true, 
    'query_var' => true,
    'rewrite' => true,
    'capability_type' => 'post',
    'has_archive' => true, 
    'hierarchical' => false,
    'menu_position' => null,
    'supports' => array('title','editor', 'page-attributes')
  ); 
  register_post_type('cmm',$args);
  
  //Register a new taxonomy. Used to allow the user to add its own menus.
  register_taxonomy("cmm_menus", array("cmm"), array("hierarchical" => true, "label" => "Menus", "singular_label" => "Menus", "rewrite" => true, "show_in_nav_menus" => false, "show_ui" => true));  
}

//Customize the menu items list
add_filter("manage_edit-cmm_columns", "cmm_menus_edit_columns");
add_action("manage_posts_custom_column",  "cmm_menus_custom_columns");

function cmm_menus_edit_columns($columns){
		//The menu items list will only have two columns: Menu Item and Menu
		$columns = array(
			"cb" => "<input type=\"checkbox\" />",
			"title" => "Menu Item",
			"cmm_menu" => "Menu",
		);

		return $columns;
}

function cmm_menus_custom_columns($column){
		global $post;
		//On the menu item editing page, display the existing menus
		switch ($column)
		{
			case "cmm_menu":
				echo get_the_term_list($post->ID, 'cmm_menus', '', ', ','');
				break;
		}
}


//Customize the admin messages displayed upon menu item addition, delete or update
add_filter('post_updated_messages', 'cmm_updated_messages');
function cmm_updated_messages( $messages ) {
  global $post, $post_ID;

  $messages['cmm'] = array(
    0 => '', // Unused. Messages start at index 1.
    1 => sprintf( __('Menu item updated. <a href="%s">View menu item</a>'), esc_url( get_permalink($post_ID) ) ),
    2 => __('Custom field updated.'),
    3 => __('Custom field deleted.'),
    4 => __('Menu item updated.'),
    /* translators: %s: date and time of the revision */
    5 => isset($_GET['revision']) ? sprintf( __('Menu item restored to revision from %s'), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
    6 => sprintf( __('Menu item published. <a href="%s">View menu item</a>'), esc_url( get_permalink($post_ID) ) ),
    7 => __('Menu item saved.'),
    8 => sprintf( __('Menu item submitted. <a target="_blank" href="%s">Preview menu item</a>'), esc_url( add_query_arg( 'preview', 'true', get_permalink($post_ID) ) ) ),
    9 => sprintf( __('Menu item scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview menu item</a>'),
      // translators: Publish box date format, see http://php.net/date
      date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ), esc_url( get_permalink($post_ID) ) ),
    10 => sprintf( __('Menu item draft updated. <a target="_blank" href="%s">Preview menu item</a>'), esc_url( add_query_arg( 'preview', 'true', get_permalink($post_ID) ) ) ),
  );

  return $messages;
}

//Add custom fields to the CSS CSS Mega Menu page
//Specify a list of custom options
$cmm_custom_meta_boxes = array(
	"cmm_column_width" => array(
	"name" => "cmm_column_width",
	"type" => "select",
	"options" => array("nodrop", "1column", "2columns", "3columns", "4columns", "5columns", "fullwidth"),
	"std" => "nodrop",
	"label" => "",
	"title" => "Dropdown area width",
	"description" => "Specify the width of the dropdown section."), 

	"cmm_menu_item_link" => array(
	"name" => "cmm_menu_item_link",
	"type" => "input",
	"std" => "",
	"label" => "",
	"title" => "Menu item link",
	"description" => "Complete URL of the page that should be opened when the menu item is clicked. Leave empty if no page should be opened upon click.<br />Example: http://www.yoursite.com/somepage.html"), 

	"cmm_menu_item_alignement" => array(
	"name" => "cmm_menu_item_alignement",
	"type" => "checkbox",
	"std" => "",
	"label" => "Right-aligned menu item",
	"title" => "Menu item alignement",
	"description" => "Check this box to align the menu item to the right side of the menu bar."), 
);

//Create the meta boxes for the custom options. Fill their content if defined or use the specified standard value (std field in the cmm_custom_meta_boxes above)
function cmm_meta_boxes() {
	global $post, $cmm_custom_meta_boxes;
 
	if(!empty($cmm_custom_meta_boxes)){
		foreach($cmm_custom_meta_boxes as $meta_box) {
			$meta_box_value = get_post_meta($post->ID, $meta_box['name'].'_value', true);
			 
			if($meta_box_value == "")
			$meta_box_value = $meta_box['std'];
			 
			echo'<input type="hidden" name="'.$meta_box['name'].'_noncename" id="'.$meta_box['name'].'_noncename" value="'.wp_create_nonce( plugin_basename(__FILE__) ).'" />';
			 
			echo'<h4>'.$meta_box['title'].'</h4>';
			 
			echo'<p><label for="'.$meta_box['name'].'_value">'.$meta_box['description'].'</label></p>';
			switch ($meta_box['type']) {
				case "input":
					echo'<input type="text" name="'.$meta_box['name'].'_value" value="'.$meta_box_value.'" size="55" />';
					break;
				case "checkbox":
					if($meta_box_value != "") {
						$checked =  "checked=\"checked\"";
					} else {
						$checked = "";
					}
					echo '<input type="checkbox" name="'.$meta_box['name'].'_value" value="'.$meta_box['name'].'" '. $checked .' />&nbsp;'.$meta_box["label"];
					break;
				case "select":
					if(!empty($meta_box['options'])){
						echo '<select name="'.$meta_box['name'].'_value">';
							foreach ($meta_box['options'] as $option) {
								if ( $meta_box_value == $option) { 
									$s = ' selected="selected"';
								} elseif ($option == $meta_box['std']) { 
									$s = ' selected="selected"';
								} else {
									$s = '';
								}
								echo '<option '. $s .'>'. $option .'</option>';
							}
						echo '</select>';
					}
					break;	
				default:
					echo "<p>Wrong meta box type selection!</p>";
					break;
			}
			 
		}
	}
}

//Create the meta box at the right moment
function cmm_create_meta_box() {
	global $theme_name;
	if ( function_exists('add_meta_box') ) {
		add_meta_box( 'cmm-meta-boxes', 'Menu item configuration options', 'cmm_meta_boxes', 'cmm', 'normal', 'high' );
	}
}

//Prepare and save custom field data
function cmm_save_postdata( $post_id ) {
	global $post, $cmm_custom_meta_boxes;

	if(!empty($cmm_custom_meta_boxes)){
		foreach($cmm_custom_meta_boxes as $meta_box) {
			if ( !wp_verify_nonce( $_POST[$meta_box['name'].'_noncename'], plugin_basename(__FILE__) )) {
				return $post_id;
			}
	 
			if ( 'cmm' == $_POST['post_type'] ) {
				if ( !current_user_can( 'edit_page', $post_id ))
					return $post_id;
			} else {
				if ( !current_user_can( 'edit_post', $post_id ))
					return $post_id;
			}
			 
			switch ($meta_box['type']) {
				case "input":
					$data = $_POST[$meta_box['name'].'_value'];
					break;
				case "checkbox":
					if($_POST['cmm_menu_item_alignement_value']) {
						$data = 'checked';
					} else {
						$data = '';
					}
					break;
				case "select":
					$data = $_POST[$meta_box['name'].'_value'];
					break;
				default:
					break;
			}
			 
			if(get_post_meta($post_id, $meta_box['name'].'_value') == "")
				add_post_meta($post_id, $meta_box['name'].'_value', $data, true);
			elseif($data != get_post_meta($post_id, $meta_box['name'].'_value', true))
				update_post_meta($post_id, $meta_box['name'].'_value', $data);
			elseif($data == "")
				delete_post_meta($post_id, $meta_box['name'].'_value', get_post_meta($post_id, $meta_box['name'].'_value', true));
		}
	}
}

//Add actions for creating the meta boxes and saving their content
add_action('admin_menu', 'cmm_create_meta_box');
add_action('save_post', 'cmm_save_postdata');


add_action ( 'cmm_menus_add_form_fields', 'cmm_additional_options');
add_action ( 'cmm_menus_edit_form', 'cmm_additional_options');
add_action ( 'edited_cmm_menus', 'cmm_save_additional_options');

function cmm_additional_options( $tag ) {
	//Display item ID
	if(isset($tag->term_taxonomy_id)) {
		echo '<span style="background: #D8FEAF; color: #34760A; padding: 5px; border: 1px solid #34760A;"><strong>Menu item ID: '.$tag->term_taxonomy_id.'</strong></span><br /><br />';
	}
	
	//Add menu type option
	$cme_menu_types = array("horizontal", "vertical-left", "vertical-right");
	$cmt = get_option( 'cmm_menu_type_value' );
	$cmm_menu_type = '';
	if ( is_array( $cmt ) && array_key_exists( $tag->term_taxonomy_id, $cmt ) ) {
		$cmm_menu_type = $cmt[$tag->term_taxonomy_id] ;
	}

    echo '<tr class="form-field">';
    echo '<th scope="row" valign="top"><label for="cmm_menu_type_value">'. _e('Menu type') .'</label></th>';
    echo '<td>';
	if(!empty($cme_menu_types)){
		echo '<select name="cmm_menu_type_value">';
		foreach ($cme_menu_types as $mt) {
				if ( $cmm_menu_type == $mt) { 
					$s = ' selected="selected"';
				} else {
					$s = '';
				}
				echo '<option '. $s .'>'. $mt .'</option>';
		}
		echo '</select>';
	}
    echo '<br /><span class="description">Specify the menu\'s appearance.</span><br /><br />';
	echo '</td>';
    echo '</tr>';


	//Add color scheme option
	$ccs = get_option( 'cmm_color_scheme_value' );
	$cmm_color_scheme = '';
	if ( is_array( $ccs ) && array_key_exists( $tag->term_taxonomy_id, $ccs ) ) {
		$cmm_color_scheme = $ccs[$tag->term_taxonomy_id] ;
	}

	$dirname = ABSPATH . 'wp-content/plugins/css-mega-menu/css/colors/';
	$dir = opendir($dirname);

    echo '<tr class="form-field">';
    echo '<th scope="row" valign="top"><label for="cmm_color_scheme_value">'. _e('Color scheme') .'</label></th>';
    echo '<td>';
	echo '<select name="cmm_color_scheme_value">';
	while(false != ($file = readdir($dir)))
	{
		if(($file != ".") && ($file != "..")) {
			$file = str_replace('.css', '', $file);
			if ( $cmm_color_scheme == $file) { 
				$s = ' selected="selected"';
			} else {
				$s = '';
			}
			echo '<option '. $s .'>'. $file .'</option>';
		}
	}
	echo '</select>';
    echo '<br /><span class="description">Specify the menu\'s color scheme.</span><br /><br />';
	echo '</td>';
    echo '</tr>';
	closedir($dir); 
	
	//Add dropdown style option
	$cme_menu_dd_style = array("dark", "light");
	$cmt = get_option( 'cmm_menu_dd_style' );
	$cmm_menu_type = '';
	if ( is_array( $cmt ) && array_key_exists( $tag->term_taxonomy_id, $cmt ) ) {
		$cmm_menu_type = $cmt[$tag->term_taxonomy_id] ;
	}

    echo '<tr class="form-field">';
    echo '<th scope="row" valign="top"><label for="cmm_menu_dd_style">'. _e('Dropdown style') .'</label></th>';
    echo '<td>';
	if(!empty($cme_menu_dd_style)){
		echo '<select name="cmm_menu_dd_style">';
		foreach ($cme_menu_dd_style as $mt) {
				if ( $cmm_menu_type == $mt) { 
					$s = ' selected="selected"';
				} else {
					$s = '';
				}
				echo '<option '. $s .'>'. $mt .'</option>';
		}
		echo '</select>';
	}
    echo '<br /><span class="description">Specify the dropdown style appearance.</span><br /><br />';
	echo '</td>';
    echo '</tr>';


}

function cmm_save_additional_options( $term_taxonomy_id ) {
	if ( isset( $_POST['cmm_menu_type_value'] ) ) {

		//load existing menu type option
		$cmm_current_menu_type = get_option( 'cmm_menu_type_value' );

		//set menu type ID to proper category ID in options array
		$cmm_current_menu_type[$term_taxonomy_id] = $_POST['cmm_menu_type_value'];

		//save the option array
		update_option( 'cmm_menu_type_value', $cmm_current_menu_type );
	}

	if ( isset( $_POST['cmm_color_scheme_value'] ) ) {

		//load existing color scheme option
		$cmm_current_color_scheme = get_option( 'cmm_color_scheme_value' );

		//set color schemer ID to proper category ID in options array
		$cmm_current_color_scheme[$term_taxonomy_id] = $_POST['cmm_color_scheme_value'];

		//save the option array
		update_option( 'cmm_color_scheme_value', $cmm_current_color_scheme );
	}
	
	if ( isset( $_POST['cmm_menu_dd_style'] ) ) {

		//load existing menu type option
		$cmm_menu_dd_style = get_option( 'cmm_menu_dd_style' );

		//set menu type ID to proper category ID in options array
		$cmm_menu_dd_style[$term_taxonomy_id] = $_POST['cmm_menu_dd_style'];

		//save the option array
		update_option( 'cmm_menu_dd_style', $cmm_menu_dd_style );
	}

}

function isIE6() {
  ereg('MSIE ([6]\.[0-9])',$_SERVER['HTTP_USER_AGENT'],$reg);
  if(!isset($reg[1])) {
    return -1;
  } else {
    return 1;
  }
}

function isIE() {
  ereg('MSIE ([0-9]\.[0-9])',$_SERVER['HTTP_USER_AGENT'],$reg);
  if(!isset($reg[1])) {
    return -1;
  } else {
    return 1;
  }
}

function build_ie6_inline_style() {
	//TDO: This line locks up IE6
	//echo '<style type="text/css">body { behavior: url("'. WP_PLUGIN_URL .'/css-mega-menu/css/csshover3.htc");}</style>';
}

require_once('css-mega-menu-shortcodes.php');
require_once('css-mega-menu-widget.php');
require_once('css-mega-menu-frontend.php');
?>