<?
/*****
 * Funktion:	Funktionsseite des Themes deals-deals-deals
 * Dateiname: 	index.php
 * Autor:		Oliver Hees / Internetallrounder.de
 * Version:		1.0
 * Erstellt:	11 Oktober, 2011  
 * 
 *****/


/*******************************
 PAGINATION
********************************
 * Retrieve or display pagination code.
 *
 * The defaults for overwriting are:
 * 'page' - Default is null (int). The current page. This function will
 *      automatically determine the value.
 * 'pages' - Default is null (int). The total number of pages. This function will
 *      automatically determine the value.
 * 'range' - Default is 3 (int). The number of page links to show before and after
 *      the current page.
 * 'gap' - Default is 3 (int). The minimum number of pages before a gap is 
 *      replaced with ellipses (...).
 * 'anchor' - Default is 1 (int). The number of links to always show at begining
 *      and end of pagination
 * 'before' - Default is '<div class="emm-paginate">' (string). The html or text 
 *      to add before the pagination links.
 * 'after' - Default is '</div>' (string). The html or text to add after the
 *      pagination links.
 * 'title' - Default is '__('Pages:')' (string). The text to display before the
 *      pagination links.
 * 'next_page' - Default is '__('&raquo;')' (string). The text to use for the 
 *      next page link.
 * 'previous_page' - Default is '__('&laquo')' (string). The text to use for the 
 *      previous page link.
 * 'echo' - Default is 1 (int). To return the code instead of echo'ing, set this
 *      to 0 (zero).
 *
 * @author Eric Martin <eric@ericmmartin.com>
 * @copyright Copyright (c) 2009, Eric Martin
 * @version 1.0
 *
 * @param array|string $args Optional. Override default arguments.
 * @return string HTML content, if not displaying.
 */
 
function emm_paginate($args = null) {
	$defaults = array(
		'page' => null, 'pages' => null, 
		'range' => 3, 'gap' => 3, 'anchor' => 1,
		'before' => '<div class="emm-paginate">', 'after' => '</div>',
		'title' => __('Pages:'),
		'nextpage' => __('&raquo;'), 'previouspage' => __('&laquo'),
		'echo' => 1
	);

	$r = wp_parse_args($args, $defaults);
	extract($r, EXTR_SKIP);

	if (!$page && !$pages) {
		global $wp_query;

		$page = get_query_var('paged');
		$page = !empty($page) ? intval($page) : 1;

		$posts_per_page = intval(get_query_var('posts_per_page'));
		$pages = intval(ceil($wp_query->found_posts / $posts_per_page));
	}
	
	$output = "";
	if ($pages > 1) {	
		$output .= "$before<span class='emm-title'>$title</span>";
		$ellipsis = "<span class='emm-gap'>...</span>";

		if ($page > 1 && !empty($previouspage)) {
			$output .= "<a href='" . get_pagenum_link($page - 1) . "' class='emm-prev'>$previouspage</a>";
		}
		
		$min_links = $range * 2 + 1;
		$block_min = min($page - $range, $pages - $min_links);
		$block_high = max($page + $range, $min_links);
		$left_gap = (($block_min - $anchor - $gap) > 0) ? true : false;
		$right_gap = (($block_high + $anchor + $gap) < $pages) ? true : false;

		if ($left_gap && !$right_gap) {
			$output .= sprintf('%s%s%s', 
				emm_paginate_loop(1, $anchor), 
				$ellipsis, 
				emm_paginate_loop($block_min, $pages, $page)
			);
		}
		else if ($left_gap && $right_gap) {
			$output .= sprintf('%s%s%s%s%s', 
				emm_paginate_loop(1, $anchor), 
				$ellipsis, 
				emm_paginate_loop($block_min, $block_high, $page), 
				$ellipsis, 
				emm_paginate_loop(($pages - $anchor + 1), $pages)
			);
		}
		else if ($right_gap && !$left_gap) {
			$output .= sprintf('%s%s%s', 
				emm_paginate_loop(1, $block_high, $page),
				$ellipsis,
				emm_paginate_loop(($pages - $anchor + 1), $pages)
			);
		}
		else {
			$output .= emm_paginate_loop(1, $pages, $page);
		}

		if ($page < $pages && !empty($nextpage)) {
			$output .= "<a href='" . get_pagenum_link($page + 1) . "' class='emm-next'>$nextpage</a>";
		}

		$output .= $after;
	}

	if ($echo) {
		echo $output;
	}

	return $output;
}

/**
 * Helper function for pagination which builds the page links.
 *
 * @access private
 *
 * @author Eric Martin <eric@ericmmartin.com>
 * @copyright Copyright (c) 2009, Eric Martin
 * @version 1.0
 *
 * @param int $start The first link page.
 * @param int $max The last link page.
 * @return int $page Optional, default is 0. The current page.
 */
function emm_paginate_loop($start, $max, $page = 0) {
	$output = "";
	for ($i = $start; $i <= $max; $i++) {
		$output .= ($page === intval($i)) 
			? "<span class='emm-page emm-current'>$i</span>" 
			: "<a href='" . get_pagenum_link($i) . "' class='emm-page'>$i</a>";
	}
	return $output;
}



/*******************************
 WIDGETS AREAS
********************************/

if ( function_exists('register_sidebar') )
register_sidebar(array(
	'name' => 'sidebar',
	'before_widget' => '<div class="rightBox">
			<div class="rightBoxTop"></div>
			<div class="rightBoxMid">',
	'after_widget' => '
			</div>
			<div class="rightBoxBottom"></div>
		</div><div class="clear-left">&nbsp;</div>',
	'before_title' => '<div class="sidebar-titel">',
	'after_title' => '</div>',
));

register_sidebar(array('name'=>'Suche',
'before_widget' => '<div id="im-fokus">',
'after_widget' => '</div>',
'before_title' => '<h2>',
'after_title' => '</h2>',
));  

/*******************************
 Feedmaske
********************************/

function feedmaske() { 
	global $post;
	$maske = get_post_meta($post->ID, 'feedmaske', true); // Auslesen Status Feedmaske
	if ($maske == 'standard') {
		include (TEMPLATEPATH . '/feedmaske/index/standard-maske.php'); //2 Spalten 4 Anzeigen
		}
		else if ($maske == 'medium') {
		include (TEMPLATEPATH . '/feedmaske/index/medium-maske.php'); // 2 Spalten 2 Anzeigen + Makotchen
		}
		else if ($maske == 'voll') {
		include (TEMPLATEPATH . '/feedmaske/index/voll-maske.php'); // 2 Spalten 1 Anzeige + Maskotchen
		}
		else if ($maske == 'off') {
		include (TEMPLATEPATH . '/feedmaske/index/off-maske.php'); // Keine Anzeige
		}
		else {
		include (TEMPLATEPATH . '/feedmaske/index/standard-maske.php'); // Keine Anzeige
	}
}

/*******************************
 Custom Fields in Rss Feed
********************************/

// Feedmonkey - WP Funktion welche den RSS Feed zum Beitrag ergänzt***
function feed_monkey( $content ) {
    global $post, $id;
    if ( !is_feed() )
        return $content;
    // Auslesen/Prüfung des Benutzerdefinierten Feldes mit dem Namen Anhang ***
    $beschreibung = get_post_meta( $post->ID, 'beschreibung', true );
    // Print custom fields ***
    if ( $beschreibung != '')
        return __(''). $beschreibung;
}
add_filter( 'the_content', 'feed_monkey' );

/*******************************
 Custom Fields / Benutzerdefinierte Felder
********************************/

//Benutzerdefiniertes Feld "Beschreibung"
function beschreibung() {
	global $post, $id;
	/* Meta Daten Abruf*/
	$beschreibung = get_post_meta($post->ID, 'beschreibung', true);
	/* Ausgabe Meta Daten */
	echo "<p>".$beschreibung."</p>";
    
}

//Benutzerdefiniertes Feld "Bild"
function bild() {
	global $post, $id;
	/* Meta Daten Abruf*/
	$bild = get_post_meta($post->ID, 'bild', true);
	/* Ausgabe Meta Daten */
	echo $bild;
    
}

//Benutzerdefiniertes Feld "Konditionen"
function konditionen() {
	global $post, $id;
	/* Meta Daten Abruf*/
	$konditionen = get_post_meta($post->ID, 'konditionen', true);
	/* Ausgabe Meta Daten */
	echo "<p>".$konditionen."</p>";
    
}

//Benutzerdefiniertes Feld "Highlight"
function highlight() {
	global $post, $id;
	/* Meta Daten Abruf*/
	$highlight = get_post_meta($post->ID, 'highlight', true);
	/* Ausgabe Meta Daten */
	echo "<p>".$highlight."</p>";
    
}

//Benutzerdefiniertes Feld "Partner Logo"
function partner_logo() {
	global $post, $id;
	/* Meta Daten Abruf*/
	$partner_logo = get_post_meta($post->ID, 'partner_logo', true);
	/* Ausgabe Meta Daten */
	echo "<img src=\"".$partner_logo."\" width=\"80\" title=\"".$post_titel."\"/>";
    
}

function alter_preis() {
	global $post;
	/* Meta Daten Abruf Alter Preis */
	$alter_preis_n = get_post_meta($post->ID, 'alter_preis', true);
	
    /* Euro Zeichen Entfernen */ 
	$alter_preis = preg_replace("/€/","",$alter_preis_n);
	
	if ($alter_preis == '') 
	{
	echo "";
	}
	else
	{
	echo "<div class=\"a_preis\">".$alter_preis." €</div>";
	}
}

function neuer_preis() {
	global $post;
	/* Meta Daten Abruf Neuer Preis */
	$neuer_preis_n = get_post_meta($post->ID, 'neuer_preis', true);
    
    /* Euro Zeichen Entfernen */ 
	$neuer_preis = preg_replace("/€/","",$neuer_preis_n);
	
	echo "<div class=\"n_preis\">".$neuer_preis." €</div>";
    
}

// Berechnung der Ersparnis anhand alter und neuer Preis
function prozente() {
	global $post;
	/* Meta Daten Abruf Alter Preis */
	$alter_preis_n = get_post_meta($post->ID, 'alter_preis', true);
	
    /* Euro Zeichen Entfernen */ 
	$alter_preis = preg_replace("/€/","",$alter_preis_n);
	
	/* Meta Daten Abruf Neuer Preis */
	$neuer_preis_n = get_post_meta($post->ID, 'neuer_preis', true);
    
    /* Euro Zeichen Entfernen */ 
	$neuer_preis = preg_replace("/€/","",$neuer_preis_n);
	
	$prozent_ergebnis = $alter_preis - $neuer_preis;
    $zergebnis = $prozent_ergebnis / $alter_preis;
    $ergebnis = $zergebnis * 100 ;
	$gerundet = round($ergebnis, 0);
	
	echo "<div class=\"c_prozente\">".$gerundet." %</div>";
	
	
}

function bereits_verkauft() {
	global $post;
	$bereits_verkauft = get_post_meta($post->ID, 'bereits_verkauft', true);
	
	if ($bereits_verkauft == '') {
		echo "--";
	}
	else {
	echo "<div class=\"b_bereits_verkauft\">".$bereits_verkauft." Stck</div>";
	}
    
}



/*******************************
Vollautomatisches Linkmaskierung
********************************/

if (!function_exists('is_admin')) {
header('Status: 403 Forbidden');
header('HTTP/1.1 403 Forbidden');
exit();
}
if (function_exists('add_shortcode')) {
add_action(
'template_redirect',
create_function(
'',
'if (preg_match("/\/outgoing\/([0-9]{1})\/(.*)\.html(\/?)$/", $_SERVER["REQUEST_URI"], $matches)) {
if (php_sapi_name() != "cgi-fcgi") {
status_header(302);
}
header(
"Location: ". sprintf(
"http://%s%s",
$matches[1] ? "www." : "",
str_rot13(html_entity_decode(urldecode(urldecode($matches[2]))))
)
);
exit();
}'
)
);
add_shortcode(
'wpaffi',
create_function(
'$attr',
'return sprintf(
"%s/outgoing/%s/%s.html",
get_bloginfo("home"),
preg_match("#^htt(.{1,2})://www\n.#", $attr["link"]) ? 1 : 0,
urlencode(urlencode(str_rot13(preg_replace("#^(?:http://|www\.)#", "", $attr["link"]))))
);'
)
);
}

// Zusammensetzen des zu Maskierenden Links
// Vorausetzung für Link: Customfile: "aff_link" sowie wpaffi
function linkiate() {
	global $post;
	$aff_link = get_post_meta( $post->ID, 'aff_link', true);
	if ($aff_link)
    echo do_shortcode('[wpaffi link="'. $aff_link .'"]');
    
}

?>